%--------------------------------------------------------------------------
% Simulation Experiment for GMM Averaging Estimator
% model = 4, the DGP S3 in the paper
%--------------------------------------------------------------------------
% Compare finite sample risks of 6 different estimators
% (1) Conservative GMM
% (2) Aggressive GMM
% (3) Pre-test (J test) model selection GMM nominal size = 0.01
% (4) Pre-test (J test) model selection GMM nominal size = 0.05
% (5) Feasible "optimal" GMM averaging  
% (6) Positive JS averaging 
% (7) Restricted JS averaging
%-------------------------------------------------------------------------- 
                     clear; close all; clc;
%--------------------------------------------------------------------------  
for model = 4

R = 10000; df = 7; Tr = 1000;

for n = [1000, 500, 250, 100, 50]
      
tic;
%--------------------------------------------------------------------------
%            Define the matrix of the location parameters
%--------------------------------------------------------------------------
alpha1  = [1 3 5 7]*pi/4;
alpha2  = [1 3]*pi/4;
N1      = length(alpha1);
N2      = length(alpha2);
D1      = [sin(alpha1') cos(alpha1')];
D2      = [sin(alpha2') cos(alpha2')];
r       = 0:0.02:1;
d1      = kron(kron(kron(D2(:,1),D2(:,1)),D2(:,1)),D1(:,1)); 
d2      = kron(kron(kron(D2(:,1),D2(:,1)),D2(:,1)),D1(:,2)); 
d3      = kron(kron(kron(D2(:,1),D2(:,1)),D2(:,2)),ones(N1,1));  
d4      = kron(kron(D2(:,1),D2(:,1)),ones(N1*N2,1)); 
d5      = kron(D2(:,1),ones(N1*N2*N2,1));

D3      = [0;1];
c1      = [kron(ones(2,1),D3) kron(D3,ones(2,1))];
c2      = [kron(ones(2,1),c1) kron(D3,ones(4,1))];
c3      = [kron(ones(2,1),c2) kron(D3,ones(8,1))];
c4      = [kron(ones(2,1),c3) kron(D3,ones(16,1))];

c4(1,:) = [];

e       = [d1 d2 d3 d4 d5;c4];
d       = [kron(r',e(:,1)) kron(r',e(:,2)) kron(r',e(:,3)) kron(r',e(:,4)) kron(r',e(:,5))]; 
M       = length(d);
%--------------------------------------------------------------------------   
ST      = zeros(M,6); 
%--------------------------------------------------------------------------
% Create matrices to store finite sample risks, bias, variances of 7 estimators
%--------------------------------------------------------------------------
 AMSE_con  = ST; AMSE_agg  = ST; AMSE_tst1  = ST; AMSE_tst2  = ST; AMSE_fea  = ST; AMSE_JS1  = ST; AMSE_JS2  = ST; 
 
 TAMSE_con = ST; TAMSE_agg = ST; TAMSE_tst1 = ST; TAMSE_tst2 = ST; TAMSE_fea = ST; TAMSE_JS1 = ST; TAMSE_JS2 = ST; 
 
 BIAS_con = ST; BIAS_agg = ST; BIAS_tst1 = ST; BIAS_tst2 = ST; BIAS_fea = ST; BIAS_JS1 = ST; BIAS_JS2 = ST; 
 
 VARI_con = ST; VARI_agg = ST; VARI_tst1 = ST; VARI_tst2 = ST; VARI_fea = ST; VARI_JS1 = ST; VARI_JS2 = ST;
 
 W_fea_M  = zeros(M,1);  W_fea_Q1  = zeros(M,1);  W_fea_Q2  = zeros(M,1); 
 
 W_JS1_M  = zeros(M,1);  W_JS1_Q1  = zeros(M,1);  W_JS1_Q2  = zeros(M,1); 
 
 W_JS2_M  = zeros(M,1);  W_JS2_Q1  = zeros(M,1);  W_JS2_Q2  = zeros(M,1);
%--------------------------------------------------------------------------
for m = 1:M
 %-------------------------------------------------------------------------
 %      Create matrices to store estimates of 6 estimators 
 %-------------------------------------------------------------------------
   STR = zeros(R,6); 
   
   Para_con = STR;  Para_agg = STR;  Para_tst1 = STR; Para_tst2 = STR; 
   
   Para_fea = STR;  Para_JS1 = STR;  Para_JS2  = STR;
   
   W1 = zeros(R,1); W2 = zeros(R,1); W3 = zeros(R,1);
   %-----------------------------------------------------------------------
   % Set seed of generating simulated data
   %-----------------------------------------------------------------------
   tseed  = 19800712; randn('seed',tseed);
   %-----------------------------------------------------------------------
   % Set true parameter values of model
   %-----------------------------------------------------------------------
   theta = 2.5*ones(6,1); beta = .50; delta = d(m,:)';
   %-----------------------------------------------------------------------
   for r = 1:R
        %------------------------------------------------------------------
        % Generate data set
        %------------------------------------------------------------------
        Data = data(theta,beta,delta,n,model);
        %------------------------------------------------------------------
        % (1) conservative GMM
        %------------------------------------------------------------------
        [con1,~,Weitcon1,Sigma11] = gmm4(Data,1);
        Para_con(r,:) = con1';
        %------------------------------------------------------------------
        % (2) pre-test (J test) model selection GMM, using full GMM
        %------------------------------------------------------------------
        [full1,fvalfull1,Weitfull1,Sigma21] = gmm4(Data,2);
        Para_agg(r,:) = full1';
        if fvalfull1  < chi2inv(.99,df); p_test_est = full1; 
        else
           p_test_est = con1;
        end
        Para_tst1(r,:) = p_test_est';
        
        if fvalfull1  < chi2inv(.95,df); p_test_est = full1; 
        else
           p_test_est = con1;
        end
        Para_tst2(r,:) = p_test_est';
        %------------------------------------------------------------------
        % (3) Feasible "optimal" GMM averaging 
        %------------------------------------------------------------------ 
        C             = trace(Sigma11 - Sigma21); 
        l             = max(abs(eig(Sigma11 - Sigma21)));  
        wfea          = C/(n*(con1-full1)'*(con1-full1)+C);
        fea           = (1 - wfea)*con1 + wfea*full1;
        Para_fea(r,:) = fea; 
        W1(r,:)       = wfea;
        %------------------------------------------------------------------
        % (4) Positive James Stein
        %------------------------------------------------------------------
        wave  = 1 - (C - 2*l)/(n*(full1-con1)'*(full1-con1));
        wjs1  = 1 - max(wave,0); 
        JS1_e = (1 - wjs1)*con1 + wjs1*full1;
        Para_JS1(r,:) = JS1_e';
        W2(r,:)       = wjs1;
        %------------------------------------------------------------------
        % (5) Restricted James Stein
        %------------------------------------------------------------------
        wjs2  = max(wjs1,0); 
        JS2_e = (1 - wjs2)*con1 + wjs2*full1;
        Para_JS2(r,:) = JS2_e';
        W3(r,:)       = wjs2;
    %----------------------------------------------------------------------
    end
    %----------------------------------------------------------------------
    % Compute the bias, variance and finit sample risk of the estimators
    %---------------------------------------------------------------------- 
    Para_est      = Para_con;

    BIAS_con(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_con(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_con(m,:) = mean(MSE);  
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_con(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Para_est      = Para_agg;

    BIAS_agg(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_agg(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_agg(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_agg(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Para_est      = Para_tst1; 

    BIAS_tst1(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_tst1(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_tst1(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_tst1(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Para_est      = Para_tst2; 

    BIAS_tst2(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_tst2(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_tst2(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_tst2(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
     
    
    Para_est      = Para_fea; 

    BIAS_fea(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_fea(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_fea(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_fea(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Para_est      = Para_JS1; 

    BIAS_JS1(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_JS1(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_JS1(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_JS1(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Para_est      = Para_JS2; 

    BIAS_JS2(m,:) = mean(Para_est - ones(R,1)*theta');

    VARI_JS2(m,:) = mean((Para_est - ones(R,1)*mean(Para_est)).*(Para_est - ones(R,1)*mean(Para_est)));
    
    MSE           = n*(Para_est - ones(R,1)*theta').*(Para_est - ones(R,1)*theta');
    
    AMSE_JS2(m,:) = mean(MSE);
    
    D_MSE         = (MSE > Tr); 
    
    TAMSE_JS2(m,:) = mean(MSE.*(1 - D_MSE) + Tr*D_MSE);   
    
    
    Weight        = W1;

    W_fea_M(m,:)  = median(Weight);  
    
    W_fea_Q1(m,:) = quantile(Weight,0.05);
    
    W_fea_Q2(m,:) = quantile(Weight,0.95);
    
    Weight        = W2;

    W_JS1_M(m,:)  = median(Weight);  
    
    W_JS1_Q1(m,:) = quantile(Weight,0.05);
    
    W_JS1_Q2(m,:) = quantile(Weight,0.95);
    
    Weight        = W3;

    W_JS2_M(m,:)  = median(Weight);  
    
    W_JS2_Q1(m,:) = quantile(Weight,0.05);
    
    W_JS2_Q2(m,:) = quantile(Weight,0.95);
end
%--------------------------------------------------------------------------
%                  Save the results
%-------------------------------------------------------------------------- 
t_c  = toc;

name = 'GMM_Averaging_Results_%d_%d.mat';

save(sprintf(name,model,n),'BIAS_con','VARI_con','AMSE_con','TAMSE_con','BIAS_agg','VARI_agg','AMSE_agg','TAMSE_agg','BIAS_tst1','VARI_tst1','AMSE_tst1','TAMSE_tst1','BIAS_tst2','VARI_tst2','AMSE_tst2','TAMSE_tst2','BIAS_fea','VARI_fea','AMSE_fea','TAMSE_fea','BIAS_JS1','VARI_JS1','AMSE_JS1','TAMSE_JS1','BIAS_JS2','VARI_JS2','AMSE_JS2','TAMSE_JS2','W_fea_M','W_fea_Q1','W_fea_Q2','W_JS1_M','W_JS1_Q1','W_JS1_Q2','W_JS2_M','W_JS2_Q1','W_JS2_Q2','t_c'); 

end

end

